@@ -1,5 +1,5 @@
-__version__ = '3.0.40'
-
+# coding=utf-8
+__version__ = '4.0.7'
 import logging
 
 logging.basicConfig()
@@ -17,8 +17,6 @@ from .challenge_solution import *
 from .challenge_results import *
 from .cie_concrete import *
 
-from .runner import dt_challenges_evaluator
-
 from .make_readmes import make_readmes_main as make_readmes_main
 from .make_readme_templates import make_readmes_templates_main
-from .runner_local import runner_local_main
+
