@@ -1,13 +1,17 @@
+# coding=utf-8
 from collections import namedtuple
-from datetime import datetime
+from dataclasses import dataclass
+from datetime import datetime, date
+from typing import *
 
 import yaml
-from duckietown_challenges import InvalidConfiguration
-from duckietown_challenges.utils import indent, safe_yaml_dump
+from networkx import DiGraph, ancestors
 
+from duckietown_challenges.cmd_submit_build import parse_complete_tag
 from . import dclogger
 from .challenges_constants import ChallengesConstants
-from .utils import raise_wrapped, check_isinstance, wrap_config_reader2
+from .exceptions import InvalidConfiguration
+from .utils import indent, safe_yaml_dump, check_isinstance, wrap_config_reader2
 
 
 class InvalidChallengeDescription(Exception):
@@ -25,47 +29,152 @@ ALLOWED_CONDITION_TRIGGERS = ChallengesConstants.ALLOWED_JOB_STATUS
 allowed_permissions = ['snoop', 'change', 'moderate', 'grant']
 
 
-class ChallengeStep(object):
-    def __init__(self, name, title, description, evaluation_parameters,
-                 features_required, timeout):
-        self.name = name
-        self.title = title
-        self.description = description
-        check_isinstance(evaluation_parameters, EvaluationParameters)
-        self.evaluation_parameters = evaluation_parameters
-        check_isinstance(features_required, dict)
-        self.features_required = features_required
-        self.timeout = timeout
+@dataclass(repr=False)
+class Build:
+    context: str
+    dockerfile: str
+    args: Dict[str, Any]
+
+    #
+    # def __init__(self, context, dockerfile, args):
+    #     self.context = context
+    #     self.dockerfile = dockerfile
+    #     self.args = args
+
+    def __repr__(self):
+        return nice_repr(self)
 
     def as_dict(self):
-        data = {}
-        data['title'] = self.title
-        data['description'] = self.description
-        data['evaluation_parameters'] = self.evaluation_parameters.as_dict()
-        data['features_required'] = self.features_required
-        data['timeout'] = self.timeout
-        return data
+        return dict(context=self.context, dockerfile=self.dockerfile, args=self.args)
 
+    @classmethod
+    def from_yaml(cls, d0):
+        if not isinstance(d0, dict):
+            msg = 'Expected dict, got %s' % d0.__repr__()
+            raise ValueError(msg)
+        d = dict(**d0)
+
+        context = d.pop('context', '.')
+        dockerfile = d.pop('dockerfile', 'Dockerfile')
+        args = d.pop('args', {})
+
+        if d:
+            msg = 'Extra fields: %s' % list(d0)
+            raise ValueError(msg)
+        return Build(context, dockerfile, args)
+
+
+@dataclass
+class PortDefinition:
+    external: int
+    internal: int
+
+
+@dataclass
+class ServiceDefinition:
+    image: Optional[str]
+    build: Build
+    environment: Dict[str, Any]
+    ports: List[PortDefinition]
+
+    def __repr__(self):
+        return nice_repr(self)
+
+    def equivalent(self, other):
+        if self.image != ChallengesConstants.SUBMISSION_CONTAINER_TAG:
+            br2 = parse_complete_tag(other.image)
+
+            try:
+                br1 = parse_complete_tag(self.image)
+            except ValueError as e:
+                msg = 'Could not even parse mine'
+                raise NotEquivalent(msg) from e
+
+            if br1.digest is None or br2.digest is None:
+                msg = 'No digest information, assuming different.\nself: %s\nother: %s' % (br1, br2)
+                raise NotEquivalent(msg)
+            if br1.digest != br2.digest:
+                msg = 'Different digests:\n\n  %s\n\n  %s' % (br1, br2)
+                raise NotEquivalent(msg)
+            if self.ports != other.ports:
+                msg = 'Different digests:\n\n  %s\n\n  %s' % (br1, br2)
+                raise NotEquivalent(msg)
+
+        if self.environment != other.environment:
+            msg = 'Different environments:\n\n %s\n\n  %s' % (self.environment, other.environment)
+            raise NotEquivalent(msg)
+
+    # noinspection PyArgumentList
     @classmethod
     @wrap_config_reader2
-    def from_yaml(cls, data, name):
-        title = data.pop('title')
-        description = data.pop('description')
-        evaluation_parameters = EvaluationParameters.from_yaml(data.pop('evaluation_parameters'))
-        features_required = data.pop('features_required')
-        timeout = data.pop('timeout')
+    def from_yaml(cls, d0):
+        image = d0.pop('image', None)
+        environment = d0.pop('environment', {})
+        if environment is None:
+            environment = {}
 
-        return ChallengeStep(name, title, description, evaluation_parameters,
-                             features_required, timeout=timeout)
+        if 'build' in d0:
+            build = d0.pop('build')
+            if build is not None:
+                build = Build.from_yaml(build)
+        else:
+            build = None
 
-    def update_image(self):
-        self.evaluation_parameters.update_image()
+        if build and image:
+            msg = 'Cannot specify both "build" and "image".'
+            raise ValueError(msg)
+
+        image_digest = d0.pop('image_digest', None)
+        if image_digest:
+            image = f'{image}@{image_digest}'
+
+        for k, v in list(environment.items()):
+            if '-' in k:
+                msg = 'Invalid environment variable "%s" should not contain a space.' % k
+                raise InvalidConfiguration(msg)
+
+            if isinstance(v, (int, float)):
+                environment[k] = str(v)
+            elif isinstance(v, str):
+                pass
+            elif isinstance(v, dict):
+                # interpret as tring
+                s = yaml.safe_dump(v)
+                environment[k] = s
+            else:
+                msg = 'The type %s is not allowed for environment variable "%s".' % (type(v).__name__, k)
+                raise InvalidConfiguration(msg)
+        ports_ = d0.pop('ports', [])
+        ports = []
+        for s in ports_:
+            if not ':' in s:
+                raise InvalidConfiguration(s)
+            tokens = s.split(':')
+            external = int(tokens[0])
+            internal = int(tokens[1])
+            ports.append(PortDefinition(external=external, internal=internal))
+        return ServiceDefinition(image=image, environment=environment, build=build, ports=ports)
+
+    def as_dict(self):
+
+        res = dict(image=self.image, environment=self.environment)
+
+        ports = []
+        for p in self.ports:
+            ports.append('%s:%s' % (p.external, p.internal))
+
+        if ports:
+            res['ports'] = ports
+        if self.build:
+            res['build'] = self.build.as_dict()
+        else:
+            pass
+
+        return res
 
 
-SUBMISSION_CONTAINER_TAG = 'SUBMISSION_CONTAINER'
-
-
-class EvaluationParameters(object):
+@dataclass
+class EvaluationParameters:
     """
         You can specify these fields for the docker file:
 
@@ -82,11 +191,14 @@ class EvaluationParameters(object):
                         var: var
 
     """
+    version: str
+    services: Dict[str, ServiceDefinition]
 
     def __init__(self, version, services):
         self.version = version
         self.services = services
 
+    # noinspection PyArgumentList
     @classmethod
     @wrap_config_reader2
     def from_yaml(cls, d):
@@ -110,7 +222,7 @@ class EvaluationParameters(object):
         # SUBMISSION_CONTAINER
         n = 0
         for service_definition in services.values():
-            if service_definition.image == SUBMISSION_CONTAINER_TAG:
+            if service_definition.image == ChallengesConstants.SUBMISSION_CONTAINER_TAG:
                 n += 1
         # if n == 0:
         #     msg = 'I expect one of the services to have "image: %s".' % SUBMISSION_CONTAINER_TAG
@@ -140,6 +252,38 @@ class EvaluationParameters(object):
                 raise NotEquivalent(msg)
 
 
+@dataclass
+class ChallengeStep:
+    name: str
+    title: str
+    description: str
+    evaluation_parameters: EvaluationParameters
+    features_required: Dict[str, Any]
+    timeout: float
+
+    def as_dict(self):
+        data = {}
+        data['title'] = self.title
+        data['description'] = self.description
+        data['evaluation_parameters'] = self.evaluation_parameters.as_dict()
+        data['features_required'] = self.features_required
+        data['timeout'] = self.timeout
+        return data
+
+    # noinspection PyArgumentList
+    @classmethod
+    @wrap_config_reader2
+    def from_yaml(cls, data, name):
+        title = data.pop('title')
+        description = data.pop('description')
+        evaluation_parameters = EvaluationParameters.from_yaml(data.pop('evaluation_parameters'))
+        features_required = data.pop('features_required', {})
+        timeout = data.pop('timeout')
+
+        return ChallengeStep(name, title, description, evaluation_parameters,
+                             features_required, timeout=timeout)
+
+
 class NotEquivalent(Exception):
     pass
 
@@ -149,127 +293,6 @@ def nice_repr(x):
     return '%s\n\n%s' % (K, indent(safe_yaml_dump(x.as_dict()), '   '))
 
 
-class ServiceDefinition(object):
-    def __init__(self, image, environment, image_digest, build):
-        check_isinstance(environment, dict)
-        check_isinstance(image, (unicode, str))
-        self.image = str(image)
-        self.image_digest = image_digest
-        self.environment = environment
-        self.build = build
-
-    def __repr__(self):
-        return nice_repr(self)
-
-    def equivalent(self, other):
-        if self.image != SUBMISSION_CONTAINER_TAG:
-            if self.image_digest is None or other.image_digest is None:
-                msg = 'No digest information, assuming different.\nself: %s\nother: %s' % (self, other)
-                raise NotEquivalent(msg)
-            if self.image_digest != other.image_digest:
-                msg = 'Different digests:\n\n  %s\n\n  %s' % (self.image_digest, other.image_digest)
-                raise NotEquivalent(msg)
-
-        if self.environment != other.environment:
-            msg = 'Different environments:\n\n %s\n\n  %s' % (self.environment, other.environment)
-            raise NotEquivalent(msg)
-
-    def update_image(self):
-        self.image = get_latest(self.image)
-
-    @classmethod
-    @wrap_config_reader2
-    def from_yaml(cls, d0):
-        image = d0.pop('image')
-        environment = d0.pop('environment', {})
-        if environment is None:
-            environment = {}
-
-        if 'build' in d0:
-            build = d0.pop('build')
-            if build is not None:
-                build = Build.from_yaml(build)
-        else:
-            build = None
-        image_digest = d0.pop('image_digest', None)
-
-        for k, v in list(environment.items()):
-            if '-' in k:
-                msg = 'Invalid environment variable "%s" should not contain a space.' % k
-                raise InvalidConfiguration(msg)
-
-            if isinstance(v, (int, str, unicode)):
-                pass
-            elif isinstance(v, dict):
-                # interpret as tring
-                s = yaml.safe_dump(v)
-                environment[k] = s
-            else:
-                msg = 'The type %s is not allowed for environment variable "%s".' % (type(v).__name__, k)
-                raise InvalidConfiguration(msg)
-
-        return ServiceDefinition(image, environment, image_digest, build)
-
-    def as_dict(self):
-
-        res = dict(image=self.image, environment=self.environment, image_digest=self.image_digest)
-
-        if self.build:
-            res['build'] = self.build.as_dict()
-        else:
-            pass
-
-        return res
-
-
-class Build(object):
-    def __init__(self, context, dockerfile, args):
-        self.context = context
-        self.dockerfile = dockerfile
-        self.args = args
-
-    def __repr__(self):
-        return nice_repr(self)
-
-    def as_dict(self):
-        return dict(context=self.context, dockerfile=self.dockerfile, args=self.args)
-
-    @classmethod
-    def from_yaml(cls, d0):
-        if not isinstance(d0, dict):
-            msg = 'Expected dict, got %s' % d0.__repr__()
-            raise ValueError(msg)
-        d = dict(**d0)
-
-        context = d.pop('context', '.')
-        dockerfile = d.pop('dockerfile', 'Dockerfile')
-        args = d.pop('args', {})
-
-        if d:
-            msg = 'Extra fields: %s' % list(d0)
-            raise ValueError(msg)
-        return Build(context, dockerfile, args)
-
-
-def get_latest(image_name):
-    if '@' in image_name:
-        msg = 'The image %r already has a qualified hash. Not updating.' % image_name
-        dclogger.warning(msg)
-        return
-    import docker
-    client = docker.from_env()
-    dclogger.info('Finding latest version of %s' % image_name)
-    image = client.images.get(image_name)
-    if ':' in image_name:
-        # remove tag
-        image_name_no_tag = image_name[:image_name.index(':')]
-        image_name = image_name_no_tag
-
-    fq = image_name + '@' + image.id
-    dclogger.info('updated %s -> %r' % (image_name, fq))
-    return fq
-
-
 Transition = namedtuple('Transition', 'first condition second')
 
 
@@ -277,15 +300,41 @@ class InvalidSteps(Exception):
     pass
 
 
-class ChallengeTransitions(object):
-    def __init__(self, transitions, steps):
-        self.transitions = []
-        self.steps = steps
+@dataclass(repr=False)
+class ChallengeTransitions:
+    steps: List[str]
+    transitions: List[Transition]
+
+    @staticmethod
+    def steps_from_transitions(transitions):
+        steps = set()
         for first, condition, second in transitions:
-            assert first == STATE_START or first in self.steps, first
-            assert second in [STATE_ERROR, STATE_FAILED, STATE_SUCCESS] or second in self.steps, second
+            if first not in [STATE_ERROR, STATE_FAILED, STATE_SUCCESS]:
+                steps.add(first)
+            if second not in [STATE_ERROR, STATE_FAILED, STATE_SUCCESS]:
+                steps.add(second)
+
+        return steps
+
+    @staticmethod
+    def from_steps_transitions(steps: List[str], transitions_str: List[List[str]]):
+        transitions = []
+        for first, condition, second in transitions_str:
+            assert first == STATE_START or first in steps, first
+            assert second in [STATE_ERROR, STATE_FAILED, STATE_SUCCESS] or second in steps, second
             assert condition in ALLOWED_CONDITION_TRIGGERS, condition
-            self.transitions.append(Transition(first, condition, second))
+            transitions.append(Transition(first, condition, second))
+        return ChallengeTransitions(steps, transitions)
+
+    #
+    # def __init__(self, transitions, steps):
+    #     self.transitions = []
+    #     self.steps = steps
+    #     for first, condition, second in self.transitions:
+    #         assert first == STATE_START or first in self.steps, first
+    #         assert second in [STATE_ERROR, STATE_FAILED, STATE_SUCCESS] or second in self.steps, second
+    #         assert condition in ALLOWED_CONDITION_TRIGGERS, condition
+    #         self.transitions.append(Transition(first, condition, second))
 
     def as_list(self):
         res = []
@@ -310,23 +359,41 @@ class ChallengeTransitions(object):
                               (first, condition, second))
         return ts
 
-    def get_next_steps(self, status):
-        """ status is a dictionary from step ID to
-            status.
+    def top_ordered(self):
+        _G = self.get_graph()  # XXX
+        return list(self.steps)
+
+    def get_graph(self):
+        G = DiGraph()
+        for first, condition, second in self.transitions:
+            G.add_edge(first, second)
+        return G
+
+    def get_precs(self, x):
+        G = self.get_graph()
+        res = list(ancestors(G, x))
+        # print('precs of %s: %s' % (x, res))
+        return res
+
+    def get_next_steps(self, status: Dict[str, str],
+                       step2age=None) -> Tuple[bool, Optional[str], List[str]]:
+        """ status is a dictionary from step name to status.
 
             It contains at the beginning
 
                 START: success
 
-            Returns a list of steps.
-
-            If the list is empty, then we are done
+            Returns:
+                 bool (completE)
+                 optional status:  ['error', 'failed', 'success']
+                 a list of steps to activate next
 
         """
-        dclogger.info('Received status = %s' % status)
+        CS = ChallengesConstants
+        # dclogger.info('Received status = %s' % status)
         assert isinstance(status, dict)
         assert STATE_START in status
-        assert status[STATE_START] == 'success'
+        assert status[STATE_START] == CS.STATUS_JOB_SUCCESS
         status = dict(**status)
         for k, ks in list(status.items()):
             if k != STATE_START and k not in self.steps:
@@ -338,29 +405,117 @@ class ChallengeTransitions(object):
                 dclogger.error(msg)
                 status.pop(k)
 
+            # timeout or aborted or host error = like it never happened
+            if ks in [CS.STATUS_JOB_TIMEOUT, CS.STATUS_JOB_ABORTED, CS.STATUS_JOB_HOST_ERROR]:
+                status.pop(k)
+
+        # make sure that the steps in which they depend are ok
+
+        def predecessors_success(_):
+            precs = self.get_precs(_)
+            its_age = step2age.get(_, -1) if step2age else 0
+            for k2 in precs:
+                pred_age = step2age.get(k2, -1) if step2age else 0
+                # dclogger.debug('%s %s %s %s' % (_, its_age, k2, pred_age))
+                if pred_age > its_age:
+                    # dclogger.debug('Its depedency is younger')
+                    return False
+                if k2 not in status or status[k2] != CS.STATUS_JOB_SUCCESS:
+                    return False
+            return True
+
         to_activate = []
         for t in self.transitions:
-            if t.first in status and status[t.first] == t.condition:
-                dclogger.debug('Transition %s is activated' % str(t))
+            if t.first in status and status[t.first] == t.condition and predecessors_success(t.first):
+                # dclogger.debug('Transition %s is activated' % str(t))
 
-                like_it_does_not_exist = [ChallengesConstants.STATUS_ABORTED]
-                if t.second in status and status[t.second] not in like_it_does_not_exist:
-                    dclogger.debug('Second %s already activated (and in %s)' % (t.second, status[t.second]))
+                like_it_does_not_exist = [ChallengesConstants.STATUS_JOB_ABORTED]
+                if t.second in status and status[t.second] not in like_it_does_not_exist and \
+                        predecessors_success(t.second):
+                    # dclogger.debug('Second %s already activated (and in %s)' % (t.second, status[t.second]))
+                    pass
                 else:
                     if t.second in [STATE_ERROR, STATE_FAILED, STATE_SUCCESS]:
-                        dclogger.debug('Finishing here')
+                        # dclogger.debug('Finishing here')
                         return True, t.second.lower(), []
                     else:
 
                         to_activate.append(t.second)
 
-        dclogger.debug('Incomplete; need to do: %s' % to_activate)
+        # dclogger.debug('Incomplete; need to do: %s' % to_activate)
         return False, None, to_activate
 
 
-class Scoring(object):
-    def __init__(self, scores):
-        self.scores = scores
+class Score:
+    HIGHER_IS_BETTER = 'higher-is-better'
+    LOWER_IS_BETTER = 'lower-is-better'
+    ALLOWED = [HIGHER_IS_BETTER, LOWER_IS_BETTER]
+
+    def __init__(self, name, description, order, discretization, short):
+        if description == 'descending':
+            order = Score.HIGHER_IS_BETTER
+
+        if description == 'ascending':
+            order = Score.LOWER_IS_BETTER
+
+        if order not in Score.ALLOWED:
+            msg = 'Invalid value %s' % order
+            raise ValueError(msg)
+
+        if discretization is not None:
+            discretization = float(discretization)
+            if discretization <= 0:
+                msg = 'Need a strictly positive discretization: %s' % discretization
+                raise ValueError(msg)
+        self.name = name
+        self.description = description
+        self.order = order
+        self.discretization = discretization
+        self.short = short
+
+    def __repr__(self):
+        return nice_repr(self)
+
+    def as_dict(self):
+        return dict(description=self.description, name=self.name, order=self.order,
+                    discretization=self.discretization, short=self.short)
+
+    @classmethod
+    def from_yaml(cls, data0):
+        try:
+            if not isinstance(data0, dict):
+                msg = 'Expected dict, got %s' % type(data0).__name__
+                raise InvalidChallengeDescription(msg)
+
+            data = dict(**data0)
+            short = data.pop('short', None)
+            name = data.pop('name')
+            description = data.pop('description', None)
+            order = data.pop('order', Score.HIGHER_IS_BETTER)
+            # TODO: remove
+            if order == 'ascending':
+                order = Score.HIGHER_IS_BETTER
+            if order == 'descending':
+                order = Score.LOWER_IS_BETTER
+            if order not in Score.ALLOWED:
+                msg = 'Invalid value "%s" not in %s.' % (order, Score.ALLOWED)
+                raise InvalidChallengeDescription(msg)
+
+            discretization = data.pop('discretization', None)
+
+            if data:
+                msg = 'Extra keys in configuration file: %s' % list(data)
+                raise InvalidChallengeDescription(msg)
+
+            return Score(name=name, description=description, order=order, discretization=discretization, short=short)
+        except KeyError as e:
+            msg = 'Missing config %s' % e
+            raise InvalidChallengeDescription(msg) from e
+
+
+@dataclass(repr=False)
+class Scoring:
+    scores: List[Score]
 
     def as_dict(self):
         scores = [_.as_dict() for _ in self.scores]
@@ -391,66 +546,21 @@ class Scoring(object):
 
         except KeyError as e:
             msg = 'Missing config %s' % e
-            raise_wrapped(InvalidChallengeDescription, e, msg)
+            raise InvalidChallengeDescription(msg) from e
 
 
-class Score(object):
-    HIGHER_IS_BETTER = 'higher-is-better'
-    LOWER_IS_BETTER = 'lower-is-better'
-    ALLOWED = [HIGHER_IS_BETTER, LOWER_IS_BETTER]
+class ChallengeDescription:
+    name: str
+    title: str
+    description: str
+    protocol: str
+    date_open: datetime
+    date_close: datetime
+    steps: Dict[str, ChallengeStep]
+    roles: Any
+    ct: ChallengeTransitions
+    scoring: Scoring
 
-    def __init__(self, name, description, order):
-        if description == 'descending':
-            order = Score.HIGHER_IS_BETTER
-
-        if description == 'ascending':
-            order = Score.LOWER_IS_BETTER
-
-        if not order in Score.ALLOWED:
-            msg = 'Invalid value %s' % order
-            raise ValueError(msg)
-
-        self.name = name
-        self.description = description
-        self.order = order
-
-    def __repr__(self):
-        return nice_repr(self)
-
-    def as_dict(self):
-        return dict(description=self.description, name=self.name, order=self.order)
-
-    @classmethod
-    def from_yaml(cls, data0):
-        try:
-            if not isinstance(data0, dict):
-                msg = 'Expected dict, got %s' % type(data0).__name__
-                raise InvalidChallengeDescription(msg)
-
-            data = dict(**data0)
-            name = data.pop('name')
-            description = data.pop('description', None)
-            order = data.pop('order', Score.HIGHER_IS_BETTER)
-            # TODO: remove
-            if order == 'ascending':
-                order = Score.HIGHER_IS_BETTER
-            if order == 'descending':
-                order = Score.LOWER_IS_BETTER
-            if not order in Score.ALLOWED:
-                msg = 'Invalid value "%s" not in %s.' % (order, Score.ALLOWED)
-                raise InvalidChallengeDescription(msg)
-
-            if data:
-                msg = 'Extra keys in configuration file: %s' % list(data)
-                raise InvalidChallengeDescription(msg)
-
-            return Score(name, description, order)
-        except KeyError as e:
-            msg = 'Missing config %s' % e
-            raise_wrapped(InvalidChallengeDescription, e, msg)
-
-
-class ChallengeDescription(object):
     def __init__(self, name, title, description, protocol,
                  date_open, date_close, steps, roles, transitions, tags, scoring):
         self.name = name
@@ -478,7 +588,7 @@ class ChallengeDescription(object):
                 raise InvalidChallengeDescription(msg)
 
         self.first_step = None
-        self.ct = ChallengeTransitions(transitions, list(self.steps))
+        self.ct = ChallengeTransitions.from_steps_transitions(list(self.steps), transitions)
 
     def get_steps(self):
         return self.steps
@@ -486,6 +596,7 @@ class ChallengeDescription(object):
     def get_next_steps(self, status):
         return self.ct.get_next_steps(status)
 
+    # noinspection PyArgumentList
     @classmethod
     @wrap_config_reader2
     def from_yaml(cls, data):
@@ -543,40 +654,49 @@ def interpret_date(d):
         return d
     if isinstance(d, datetime):
         return d
-    if isinstance(d, (str, unicode)):
+    if isinstance(d, date):
+        return datetime.combine(d, datetime.min.time())
+
+    if isinstance(d, str):
         from dateutil import parser
         return parser.parse(d)
     raise ValueError(d.__repr__())
 
 
-class SubmissionDescription(object):
-    def __init__(self, challenge_name, protocol, user_label, user_metadata, description):
-        self.challenge_name = challenge_name
-        self.protocol = protocol
-        self.user_label = user_label
-        self.user_metadata = user_metadata
-        self.description = description
+@dataclass(repr=False)
+class SubmissionDescription:
+    challenge_names: List[str]
+    protocol: List[str]
+    user_label: Optional[str]
+    user_metadata: dict
+    description: Optional[str]
 
     def __repr__(self):
         return nice_repr(self)
 
     def as_dict(self):
         return dict(protocol=self.protocol,
-                    challenge_name=self.challenge_name,
+                    challenge_names=self.challenge_names,
                     user_label=self.user_label,
                     user_metadata=self.user_metadata,
                     description=self.description)
 
+    # noinspection PyArgumentList
     @classmethod
     @wrap_config_reader2
     def from_yaml(cls, data):
         challenge_name = data.pop('challenge')
+        if isinstance(challenge_name, list):
+            challenges = challenge_name
+        else:
+            challenges = [challenge_name]
+
         protocol = data.pop('protocol')
         description = data.pop('description', None)
         user_label = data.pop('user-label', None)
         user_metadata = data.pop('user-payload', None)
 
-        return SubmissionDescription(challenge_name=challenge_name,
+        return SubmissionDescription(challenge_names=challenges,
                                      protocol=protocol,
                                      description=description,
                                      user_label=user_label,
