@@ -1,8 +1,8 @@
+# coding=utf-8
 from abc import ABCMeta, abstractmethod
 
 
-class ChallengeSolution(object):
-    __metaclass__ = ABCMeta
+class ChallengeSolution(metaclass=ABCMeta):
 
     @abstractmethod
     def run(self, cie):
