@@ -1,41 +1,56 @@
+# coding=utf-8
+import json
 import math
 import os
 import shutil
+import subprocess
 import sys
 import tempfile
 import time
 import traceback
 from collections import namedtuple
+from contextlib import contextmanager
+from dataclasses import dataclass
 
+from duckietown_challenges import ChallengesConstants
 from . import dclogger, ENV_CHALLENGE_STEP_NAME
 from .constants import CHALLENGE_DESCRIPTION_YAML, CHALLENGE_SOLUTION_OUTPUT_YAML, CHALLENGE_SOLUTION_OUTPUT_DIR, \
-    CHALLENGE_EVALUATION_OUTPUT_DIR, CHALLENGE_DESCRIPTION_DIR, ChallengeResultsStatus, CHALLENGE_PREVIOUS_STEPS_DIR, \
-    ENV_CHALLENGE_NAME
+    CHALLENGE_EVALUATION_OUTPUT_DIR, CHALLENGE_DESCRIPTION_DIR, CHALLENGE_PREVIOUS_STEPS_DIR, \
+    ENV_CHALLENGE_NAME, DEFAULT_ROOT, CHALLENGE_TMP_SUBDIR
 from .exceptions import InvalidSubmission, InvalidEvaluator, InvalidEnvironment
 from .solution_interface import ChallengeInterfaceSolution, ChallengeInterfaceEvaluator
-from .utils import raise_wrapped, d8n_make_sure_dir_exists
+from .utils import d8n_make_sure_dir_exists
 from .yaml_utils import read_yaml_file, write_yaml
+from typing import *
 
-ChallengeFile = namedtuple('ChallengeFile', 'basename from_file contents description')
+@dataclass
+class ChallengeFile:
+    basename: str
+    from_file: Optional[str]
+    contents: Optional[bytes]
+    description: str
+
+# ChallengeFile = namedtuple('ChallengeFile', 'basename from_file contents description')
 ReportedScore = namedtuple('ReportedScore', 'name value description')
 
 
+
 def check_valid_basename(s):
     pass  # TODO
 
 
-class FS(object):
+class FS:
     def __init__(self):
         self.files = {}
 
-    def add_from_data(self, basename, contents, description):
+    def add_from_data(self, basename: str, contents: bytes, description: str):
         if basename in self.files:
             msg = 'Already know %r' % basename
             raise ValueError(msg)
 
         self.files[basename] = ChallengeFile(basename, None, contents, description)
 
-    def add(self, basename, from_file, description):
+    def add(self, basename: str, from_file: str, description: str):
         if not os.path.exists(from_file):
             msg = 'The file does not exist: %s' % from_file
             raise ValueError(msg)
@@ -73,7 +88,7 @@ class ChallengeInterfaceSolutionConcrete(ChallengeInterfaceSolution):
         self.failure_declared_msg = False
 
     def get_tmp_dir(self):
-        return tempfile.mkdtemp()
+        return a_tmp_dir(self.root)
 
     def get_challenge_parameters(self):
         fn = os.path.join(self.root, CHALLENGE_DESCRIPTION_YAML)
@@ -106,14 +121,14 @@ class ChallengeInterfaceSolutionConcrete(ChallengeInterfaceSolution):
             self.solution_output_files.add(basename, from_file, description)
         except ValueError as e:
             msg = 'Invalid set_solution_output_file()'
-            raise_wrapped(InvalidSubmission, e, msg)
+            raise InvalidSubmission(msg) from e
 
-    def set_solution_output_file_from_data(self, basename, contents, description=None):
+    def set_solution_output_file_from_data(self, basename: str, contents: bytes, description=None):
         try:
             self.solution_output_files.add_from_data(basename, contents, description)
         except ValueError as e:
             msg = 'Invalid set_solution_output_file()'
-            raise_wrapped(InvalidSubmission, e, msg)
+            raise InvalidSubmission(msg) from e
 
     def info(self, s):
         dclogger.info('solution:%s' % s)
@@ -169,8 +184,8 @@ class ChallengeInterfaceSolutionConcrete(ChallengeInterfaceSolution):
         return get_completed_step_solution_file(self.root, step_name, basename)
 
 
-TIMEOUT_PREPARATION = 6000
-TIMEOUT_SOLUTION = 6000
+TIMEOUT_PREPARATION = 600
+TIMEOUT_SOLUTION = 600
 
 
 def get_completed_step_solution_files(root, step_name):
@@ -218,19 +233,31 @@ class Timeout(Exception):
 
 def wait_for_file(fn, timeout, wait):
     t0 = time.time()
+    interval_notice = 10
+    i = 0
     while not os.path.exists(fn):
         passed = int(time.time() - t0)
         to_wait = timeout - passed
-        dclogger.debug('Output %s not ready yet (%s secs passed, will wait %s secs more)' % (fn, passed, to_wait))
+        if i % interval_notice == 0:
+            dclogger.debug('Output %s not ready yet (%s secs passed, will wait %s secs more)' % (fn, passed, to_wait))
         if time.time() > t0 + timeout:
             msg = 'Timeout of %s while waiting for %s.' % (timeout, fn)
             raise Timeout(msg)
         time.sleep(wait)
+        i += 1
+
+
+def a_tmp_dir(root):
+    dirname = os.path.join(root, CHALLENGE_TMP_SUBDIR)
+    if not os.path.exists(dirname):
+        os.makedirs(dirname)
+    return tempfile.mkdtemp(dir=dirname)
 
 
 class ChallengeInterfaceEvaluatorConcrete(ChallengeInterfaceEvaluator):
 
-    def __init__(self, root='/'):
+    def __init__(self, root=DEFAULT_ROOT):
+        dclogger.info(f'ChallengeInterfaceEvaluatorConcrete root = {root}')
         self.root = root
 
         self.challenge_files = FS()  # -> ChallengeFile
@@ -244,7 +271,7 @@ class ChallengeInterfaceEvaluatorConcrete(ChallengeInterfaceEvaluator):
         self.parameters = data
 
     def get_tmp_dir(self):
-        return tempfile.mkdtemp()
+        return a_tmp_dir(self.root)
 
     # preparation
 
@@ -253,7 +280,7 @@ class ChallengeInterfaceEvaluatorConcrete(ChallengeInterfaceEvaluator):
             self.challenge_files.add(basename, from_file, description)
         except ValueError as e:
             msg = 'Invalid set_challenge_file()'
-            raise_wrapped(InvalidEvaluator, e, msg)
+            raise InvalidEvaluator(msg) from e
 
     def wait_for_solution(self):
         fn = os.path.join(self.root, CHALLENGE_SOLUTION_OUTPUT_YAML)
@@ -261,7 +288,7 @@ class ChallengeInterfaceEvaluatorConcrete(ChallengeInterfaceEvaluator):
             return wait_for_file(fn, timeout=TIMEOUT_SOLUTION, wait=1)
         except Timeout as e:
             msg = 'Time out: %s' % e
-            raise InvalidSubmission(msg)
+            raise InvalidSubmission(msg) from e
 
     def get_solution_output_dict(self):
         fn = os.path.join(self.root, CHALLENGE_SOLUTION_OUTPUT_YAML)
@@ -286,10 +313,15 @@ class ChallengeInterfaceEvaluatorConcrete(ChallengeInterfaceEvaluator):
                 raise ValueError(msg)
 
         import numpy as np
-        if type(value) is np.ndarray:
+
+        if isinstance(value, (np.float64, np.float32)):
+            value = float(value)
+
+        if isinstance(value, np.ndarray):
             msg = 'Please use regular floats and not numpy array. Invalid value for %s: %s' % (name, value)
             raise Exception(msg)
 
+        # dclogger.info('found %s %s' % (value, type(value)))
         if name in self.scores:
             msg = 'Already know score %r' % name
             raise InvalidEvaluator(msg)
@@ -301,14 +333,14 @@ class ChallengeInterfaceEvaluatorConcrete(ChallengeInterfaceEvaluator):
             self.evaluation_files.add(basename, from_file, description)
         except ValueError as e:
             msg = 'Invalid set_evaluation_file()'
-            raise_wrapped(InvalidEvaluator, e, msg)
+            raise InvalidEvaluator(msg) from e
 
     def set_evaluation_file_from_data(self, basename, contents, description=None):
         try:
             self.evaluation_files.add_from_data(basename, contents, description)
         except ValueError as e:
             msg = 'Invalid set_evaluation_file_from_data()'
-            raise_wrapped(InvalidEvaluator, e, msg)
+            raise InvalidEvaluator(msg) from e
 
     def info(self, s):
         dclogger.info('evaluation: %s' % s)
@@ -340,7 +372,7 @@ class ChallengeInterfaceEvaluatorConcrete(ChallengeInterfaceEvaluator):
         d = os.path.join(self.root, CHALLENGE_EVALUATION_OUTPUT_DIR)
         self.evaluation_files.write(d)
 
-        status = ChallengeResultsStatus.SUCCESS
+        status = ChallengesConstants.STATUS_JOB_SUCCESS
         msg = None
         scores = {}
         for k, v in self.scores.items():
@@ -442,12 +474,14 @@ SPECIAL_INVALID_EVALUATOR = 'invalid-evaluator'
 SPECIAL_INVALID_SUBMISSION = 'invalid-submission'
 
 
-def wrap_evaluator(evaluator, root='/'):
+def wrap_evaluator(evaluator, root=DEFAULT_ROOT):
     from .col_logging import setup_logging_color
     setup_logging_color()
 
+    dclogger.info(f'wrap_evaluator with root = {root}')
+
     def declare(status, message):
-        if status != ChallengeResultsStatus.SUCCESS:
+        if status != ChallengesConstants.STATUS_JOB_SUCCESS:
             msg = 'declare %s:\n%s' % (status, message)
             dclogger.error(msg)
         else:
@@ -461,10 +495,10 @@ def wrap_evaluator(evaluator, root='/'):
     try:
         try:
             evaluator.prepare(cie)
-        except BaseException as e:
-            msg = 'Preparation aborted:\n%s' % traceback.format_exc(e)
+        except (BaseException, KeyboardInterrupt) as e:
+            msg = 'Preparation aborted'
             cie.set_challenge_parameters({SPECIAL_ABORT: msg})
-            raise
+            raise Exception(msg) from e
         finally:
             cie.after_prepare()
 
@@ -482,33 +516,37 @@ def wrap_evaluator(evaluator, root='/'):
             evaluator.score(cie)
             cie.after_score()
 
+    except KeyboardInterrupt:
+        msg = 'Interrupted by user:\n%s' % traceback.format_exc()
+        declare(ChallengesConstants.STATUS_JOB_ABORTED, msg)  # TODO: aborted
+
     # failure
-    except InvalidSubmission as e:
-        msg = 'InvalidSubmission:\n%s' % traceback.format_exc(e)
-        declare(ChallengeResultsStatus.FAILED, msg)
+    except InvalidSubmission:
+        msg = 'InvalidSubmission:\n%s' % traceback.format_exc()
+        declare(ChallengesConstants.STATUS_JOB_FAILED, msg)
 
     # error of evaluator
-    except InvalidEvaluator as e:
-        msg = 'InvalidEvaluator:\n%s' % traceback.format_exc(e)
-        declare(ChallengeResultsStatus.ERROR, msg)
+    except InvalidEvaluator:
+        msg = 'InvalidEvaluator:\n%s' % traceback.format_exc()
+        declare(ChallengesConstants.STATUS_JOB_ERROR, msg)
 
     # error of environment (not distinguished so far)
 
-    except InvalidEnvironment as e:
-        msg = 'InvalidEnvironment:\n%s' % traceback.format_exc(e)
-        declare(ChallengeResultsStatus.ERROR, msg)
+    except InvalidEnvironment:
+        msg = 'InvalidEnvironment:\n%s' % traceback.format_exc()
+        declare(ChallengesConstants.STATUS_JOB_HOST_ERROR, msg)
 
     except BaseException as e:
-        msg = 'Unexpected exception:\n%s' % traceback.format_exc(e)
-        declare(ChallengeResultsStatus.ERROR, msg)
+        msg = 'Unexpected exception:\n%s' % traceback.format_exc()
+        declare(ChallengesConstants.STATUS_JOB_ERROR, msg)
 
 
-def wrap_scorer(evaluator, root='/'):
+def wrap_scorer(evaluator, root=DEFAULT_ROOT):
     from .col_logging import setup_logging_color
     setup_logging_color()
 
     def declare(status, message):
-        if status != ChallengeResultsStatus.SUCCESS:
+        if status != ChallengesConstants.STATUS_JOB_SUCCESS:
             msg = 'declare %s:\n%s' % (status, message)
             dclogger.error(msg)
         else:
@@ -525,30 +563,82 @@ def wrap_scorer(evaluator, root='/'):
         cie.after_score()
 
     # failure
-    except InvalidSubmission as e:
-        msg = 'InvalidSubmission:\n%s' % traceback.format_exc(e)
-        declare(ChallengeResultsStatus.FAILED, msg)
+    except InvalidSubmission:
+        msg = 'InvalidSubmission:\n%s' % traceback.format_exc()
+        declare(ChallengesConstants.STATUS_JOB_FAILED, msg)
 
     # error of evaluator
-    except InvalidEvaluator as e:
-        msg = 'InvalidEvaluator:\n%s' % traceback.format_exc(e)
-        declare(ChallengeResultsStatus.ERROR, msg)
+    except InvalidEvaluator:
+        msg = 'InvalidEvaluator:\n%s' % traceback.format_exc()
+        declare(ChallengesConstants.STATUS_JOB_ERROR, msg)
 
     # error of environment (not distinguished so far)
 
-    except InvalidEnvironment as e:
-        msg = 'InvalidEnvironment:\n%s' % traceback.format_exc(e)
-        declare(ChallengeResultsStatus.ERROR, msg)
+    except InvalidEnvironment:
+        msg = 'InvalidEnvironment:\n%s' % traceback.format_exc()
+        declare(ChallengesConstants.STATUS_JOB_HOST_ERROR, msg)
 
-    except BaseException as e:
-        msg = 'Unexpected exception:\n%s' % traceback.format_exc(e)
-        declare(ChallengeResultsStatus.ERROR, msg)
+    except BaseException:
+        msg = 'Unexpected exception:\n%s' % traceback.format_exc()
+        declare(ChallengesConstants.STATUS_JOB_ERROR, msg)
 
 
-def wrap_solution(solution, root='/'):
+
+
+@contextmanager
+def scoring_context(root=DEFAULT_ROOT) -> ContextManager[ChallengeInterfaceEvaluator]:
+    dclogger.info('Environment variables:\n\n' + json.dumps(dict(os.environ), indent=2))
+
+
+    from .col_logging import setup_logging_color
+    setup_logging_color()
+
+    def declare(status, message, scores=None):
+        if status != ChallengesConstants.STATUS_JOB_SUCCESS:
+            msg = 'declare %s:\n%s' % (status, message)
+            dclogger.error(msg)
+        else:
+            dclogger.info('Completed.')
+        cr = ChallengeResults(status, message, scores or {})
+        declare_challenge_results(root, cr)
+        retcode = 0 if status == ChallengesConstants.STATUS_JOB_SUCCESS else 1
+        sys.exit(retcode)
+
+    cie = ChallengeInterfaceEvaluatorConcrete(root=root)
+
+    try:
+        yield cie
+
+        scores = {}
+        for k, v in cie.scores.items():
+            scores[k] = v.value
+        declare(ChallengesConstants.STATUS_JOB_SUCCESS, None, scores)
+    # failure
+    except InvalidSubmission:
+        msg = 'InvalidSubmission:\n%s' % traceback.format_exc()
+        declare(ChallengesConstants.STATUS_JOB_FAILED, msg)
+
+    except InvalidEvaluator:
+        msg = 'InvalidEvaluator:\n%s' % traceback.format_exc()
+        declare(ChallengesConstants.STATUS_JOB_ERROR, msg)
+
+    except InvalidEnvironment:
+        msg = 'InvalidEnvironment:\n%s' % traceback.format_exc()
+        declare(ChallengesConstants.STATUS_JOB_HOST_ERROR, msg)
+
+    except SystemExit:
+        raise
+
+    except BaseException:
+        msg = 'Unexpected exception:\n%s' % traceback.format_exc()
+        declare(ChallengesConstants.STATUS_JOB_ERROR, msg)
+
+
+def wrap_solution(solution, root=DEFAULT_ROOT):
     from .col_logging import setup_logging_color
     setup_logging_color()
     cis = ChallengeInterfaceSolutionConcrete(root=root)
+    # noinspection PyBroadException
     try:
 
         try:
@@ -556,8 +646,8 @@ def wrap_solution(solution, root='/'):
             cis.get_current_step()
         except InvalidEnvironment:
             raise
-        except BaseException as e:
-            msg = 'Invalid environment: %s' % traceback.format_exc(e)
+        except BaseException:
+            msg = 'Invalid environment: %s' % traceback.format_exc()
             raise InvalidEnvironment(msg)
 
         try:
@@ -575,8 +665,8 @@ def wrap_solution(solution, root='/'):
             solution.run(cis)
         except (InvalidSubmission, InvalidEnvironment, InvalidEvaluator):
             raise
-        except BaseException as e:
-            msg = "Uncaught exception in solution:\n%s" % traceback.format_exc(e)
+        except BaseException:
+            msg = "Uncaught exception in solution:\n%s" % traceback.format_exc()
             raise InvalidSubmission(msg)
 
         if cis.failure_declared:
@@ -587,23 +677,27 @@ def wrap_solution(solution, root='/'):
             msg = 'solution_output_dict not set. Solution must use set_solution_output_dict({}).'
             raise InvalidSubmission(msg)
 
-    except InvalidEnvironment as e:
-        msg = 'InvalidEnvironment:\n%s' % traceback.format_exc(e)
+    except InvalidEnvironment:
+        msg = 'InvalidEnvironment:\n%s' % traceback.format_exc()
         cis.error(msg)
         cis.set_solution_output_dict({SPECIAL_INVALID_ENVIRONMENT: msg})
-    except InvalidEvaluator as e:
-        msg = 'InvalidEvaluator:\n%s' % traceback.format_exc(e)
+    except InvalidEvaluator:
+        msg = 'InvalidEvaluator:\n%s' % traceback.format_exc()
         cis.error(msg)
         cis.set_solution_output_dict({SPECIAL_INVALID_EVALUATOR: msg})
-    except InvalidSubmission as e:
-        msg = 'Invalid solution:\n%s' % traceback.format_exc(e)
+    except InvalidSubmission:
+        msg = 'Invalid solution:\n%s' % traceback.format_exc()
         cis.error(msg)
         cis.set_solution_output_dict({SPECIAL_INVALID_SUBMISSION: msg})
-    except BaseException as e:
-        msg = 'Uncaught exception: invalid wrap_evaluator:\n%s' % traceback.format_exc(e)
+    except BaseException:
+        msg = 'Uncaught exception: invalid wrap_evaluator:\n%s' % traceback.format_exc()
         cis.error(msg)
         cis.set_solution_output_dict({SPECIAL_INVALID_ENVIRONMENT: msg})
     finally:
         fn = os.path.join(cis.root, CHALLENGE_SOLUTION_OUTPUT_YAML)
         write_yaml(cis.solution_output_dict, fn)
         cis._write_files()
+
+    cmd = 'sync'
+    subprocess.check_call(cmd)
+    cis.info('Graceful termination of wrap_solution().')
