@@ -1,4 +1,4 @@
-
+# coding=utf-8
 class ChallengeException(Exception):
     pass
 
@@ -22,3 +22,6 @@ class InvalidEvaluator(Exception):
 class InvalidEnvironment(Exception):
     pass
 
+class AbortedByUser(Exception):
+    pass
+
