@@ -1,9 +1,10 @@
+# coding=utf-8
 import os
 
-from duckietown_challenges import read_yaml_file
+from .yaml_utils import read_yaml_file
 
 
-class ChallengeInfoLocal():
+class ChallengeInfoLocal:
     def __init__(self, challenge_name):
         self.challenge_name = challenge_name
 
